@@ -11,7 +11,7 @@ import shutil, struct
 from string import Template
 from glob import glob
 from fnmatch import fnmatch
-from . import jsonutils
+from . import jsonutils, increments
 from datetime import datetime, date
 import json
 import logging
@@ -538,7 +538,7 @@ class DBText:
     def dump_change_file(self, fn_template, change_type, new_data):
         if len(new_data) > 0:
             fn = fn_template.format(type=change_type)
-            jsonutils.dump_json_tables(new_data, fn)
+            jsonutils.dump_json_tables(new_data, fn, sort_keys=True)
 
     def convert_to_row_dicts(self, rows, colinfo):
         table_data = []
@@ -708,82 +708,10 @@ class DBText:
         origDir = fullDir + "_orig"
         if os.path.isdir(fullDir):
             shutil.move(fullDir, origDir)
-        self.write_data(writeDir) # should write to "fullDir"            
-        shutil.copytree(fullDir, fullDir + "_prereduce")
-        self.compare_and_reduce(origDir, fullDir)
+        self.write_data(writeDir) # should write to "fullDir"
+        converter = self.get_increment_converter()
+        converter.convert_to_increment(fullDir, origDir)
         
-    def find_matching_row(self, row, newRows, discard = []):
-        discardKeySet = set(discard)
-        for newRow in newRows:
-            unmatched_items = set(row) ^ set(newRow)
-            unmatched_keys = set(( k for k, v in unmatched_items ))
-            unmatched_keys -= discardKeySet
-            if len(unmatched_keys) == 0:
-                return newRow
-
-    def get_field_names_to_ignore(self):
-        return []
-    
-    def get_table_names_for_rename_check(self):
-        return []
-    
-    def get_dbdir_comparison(self, origdir, dbdir):
-        toCompare = []
-        toRemove = []
-        for root, _, files in os.walk(dbdir):
-            origroot = root.replace(dbdir, origdir)
-            for fn in files:
-                path = os.path.join(root, fn)
-                origpath = os.path.join(origroot, fn)
-                if os.path.isfile(origpath):
-                    if filecmp.cmp(origpath, path, shallow=False):
-                        toRemove.append(path)
-                    else:
-                        toCompare.append((origpath, path))
+    def get_increment_converter(self):
+        return increments.IncrementConverter(self.parse_table_file)
         
-        toReduce = []
-        renameCheck = []
-        for origFile, newFile in toCompare:
-            origRows = self.parse_table_file(origFile)
-            newRows = self.parse_table_file(newFile)
-            unmatched = []
-            for row in origRows:
-                newRow = self.find_matching_row(row, newRows, self.get_field_names_to_ignore())
-                if newRow:
-                    newRows.remove(newRow)
-                else:
-                    unmatched.append(row)
-            if len(newRows) == 0:
-                toRemove.append(newFile)
-            elif len(unmatched) == 0:
-                toReduce.append((newFile, newRows))
-            elif os.path.basename(newFile).split(".")[0] in self.get_table_names_for_rename_check():
-                renameCheck.append((newFile, unmatched, newRows))
-                        
-        toRename = self.check_for_renames(renameCheck, dbdir, toRemove, toReduce)
-        return toRemove, toReduce, toRename
-    
-    def check_for_renames(self, *args):
-        pass # hook for context-specific logic
-    
-    def compare_and_reduce(self, origdir, dbdir):
-        toRemove, toReduce, toRename = self.get_dbdir_comparison(origdir, dbdir)
-        for path in toRemove:
-            os.remove(path)
-            
-        for oldPath, newPath in toRename:
-            os.rename(oldPath, newPath)
-            
-        # clean empty directories
-        blob_dirs = [ os.path.join(dbdir, os.path.dirname(p)) for p in self.get_blob_patterns() ]
-        for d in blob_dirs + [ dbdir ]:
-            if os.path.isdir(d) and len(os.listdir(d)) == 0:
-                os.rmdir(d)
-                
-        for fn, contents in toReduce:
-            with open(fn, mode='w', errors='replace') as f:
-                for rowData in contents:
-                    f.write("ROW:+\n")
-                    for col, value in rowData:
-                        f.write('   ' + col + ": " + value + '\n')
-
